library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;

entity Provera_skidanje_zaglavlja is
	Port 
	( 
		clk		: in   STD_LOGIC;
		reset		: in	 STD_LOGIC;
		start		: in   STD_LOGIC;
		kraj		: in   STD_LOGIC;
		data_in	: in   STD_LOGIC_VECTOR (127 downto 0);
		start_out: out  STD_LOGIC;
		kraj_out : out STD_LOGIC;
		data_out	: out  STD_LOGIC_VECTOR (127 downto 0)
	);
end Provera_skidanje_zaglavlja;

architecture shema of Provera_skidanje_zaglavlja is

signal i:integer;
signal j:integer; 
component fifo_paket
	port 
	(
		clk   : IN std_logic;
		din   : IN std_logic_VECTOR(129 downto 0);
		wr_en : IN std_logic;
		rd_en : IN std_logic;
		dout  : OUT std_logic_VECTOR(129 downto 0);
		full  : OUT std_logic;
		empty : OUT std_logic
	);
end component;

component fifo_fragment
	port 
	(
		clk   : IN std_logic;
		din   : IN std_logic_VECTOR(129 downto 0);
		wr_en : IN std_logic;
		rd_en : IN std_logic;
		dout  : OUT std_logic_VECTOR(129 downto 0);
		full  : OUT std_logic;
		empty : OUT std_logic
	);
end component;

component fifo_fragment_cekanje
	port 
	(
		clk   : IN std_logic;
		din   : IN std_logic_VECTOR(127 downto 0);
		wr_en : IN std_logic;
		rd_en : IN std_logic;
		dout  : OUT std_logic_VECTOR(127 downto 0);
		full  : OUT std_logic;
		empty : OUT std_logic
	);
end component;

component prva_memorija  

	port
	( 
		wea: IN std_logic_vector (0 downto 0); 
		clka : IN std_logic; 
		clkb : IN std_logic;
		dina: IN std_logic_VECTOR  (39 downto 0) ;
		doutb: OUT std_logic_VECTOR  (39 downto 0) ;
		addra: IN std_logic_VECTOR  (9 downto 0) ;
		addrb: IN std_logic_VECTOR  (9 downto 0)
	);
end component;

component druga_memorija  

	port
	( 
		wea: IN std_logic_vector (0 downto 0); 
		clka : IN std_logic; 
		clkb : IN std_logic;
		dina: IN std_logic_VECTOR  (1151 downto 0) ;
		doutb: OUT std_logic_VECTOR  (1151 downto 0) ;
		addra: IN std_logic_VECTOR  (9 downto 0) ;
		addrb: IN std_logic_VECTOR  (9 downto 0)
	);
end component;

-- signali za potrebe fifo_paket

signal ka_fifo_paket:std_logic_vector (129 downto 0) ;
signal upis_paket:std_logic;
signal citaj_paket:std_logic;
signal procitano_paket:std_logic_vector (129 downto 0) ;
signal pun_fifo_paket:std_logic;
signal prazan_fifo_paket:std_logic;

-- signali potrebni za fifo_fragment
signal ka_fifo_fragment:std_logic_vector  (129 downto 0) ;
signal upis_fragment:std_logic;
signal citaj_fragment:std_logic;
signal procitano_fragment:std_logic_vector  (129 downto 0) ;
signal pun_fifo_fragment:std_logic;
signal prazan_fifo_fragment:std_logic;

-- signali potrebni za fifo_fragment_cekanje

signal ka_fifo_cekanje:std_logic_vector (127 downto 0) ;
signal upis_fifo_cek:std_logic;
signal dalje_fifo_cek:std_logic;
signal iz_fifo_cek:std_logic_vector (127 downto 0)   ;
signal pun_fifo_cek:std_logic;
signal prazan_fifo_cek:std_logic;

-- signali potrebni za prvu memoriju

signal upisi_u_prvu:std_logic_vector (0 downto 0):= (others => '0');
signal u_prvu_memoriju:std_logic_vector (39 downto 0) ;
signal iz_prve_memorije:std_logic_vector (39 downto 0)  ;
signal adr_mem_upis_1:std_logic_vector (9 downto 0):= (others => '0') ;
signal adr_mem_ispis_1:std_logic_vector (9 downto 0):= (others => '0') ;

-- signali potrebni za drugu memoriju

signal upisi_u_drugu:std_logic_vector (0 downto 0):= (others => '0');
signal u_drugu_memoriju:std_logic_vector (1151 downto 0) ;
signal iz_druge_memorije:std_logic_vector (1151 downto 0)  ;
signal adr_mem_upis_2:std_logic_vector (9 downto 0):= (others => '0') ;
signal adr_mem_ispis_2:std_logic_vector (9 downto 0):= (others => '0') ;

begin

za_cele_pakete: fifo_paket
	port map
	(
		clk => clk,
		din => ka_fifo_paket,
		wr_en => upis_paket,
		rd_en => citaj_paket,
		dout => procitano_paket,
		full => pun_fifo_paket,
		empty => prazan_fifo_paket
	);
	
	za_fragmente: fifo_fragment
	port map
	(
		clk => clk,
		din => ka_fifo_fragment,
		wr_en => upis_fragment,
		rd_en => citaj_fragment,
		dout => procitano_fragment,
		full => pun_fifo_fragment,
		empty => prazan_fifo_fragment
	);
	
	za_buffering_fragmente: fifo_fragment_cekanje
	port map
	(
		clk => clk,
		din => ka_fifo_cekanje,
		wr_en => upis_fifo_cek,
		rd_en => dalje_fifo_cek,
		dout => iz_fifo_cek,
		full => pun_fifo_cek,
		empty => prazan_fifo_cek
	);
	
	prva_memorija_za_fragmente: prva_memorija
	port map
	( 
		wea=>upisi_u_prvu, 
		clka=>clk, 
		clkb=>clk,
		dina=>u_prvu_memoriju,
		doutb=>iz_prve_memorije,
		addra=>adr_mem_upis_1,
		addrb=>adr_mem_ispis_1
	);
	
	druga_memorija_za_fragmente: druga_memorija
	port map
	( 
		wea=>upisi_u_drugu, 
		clka=>clk, 
		clkb=>clk,
		dina=>u_drugu_memoriju,
		doutb=>iz_druge_memorije,
		addra=>adr_mem_upis_2,
		addrb=>adr_mem_ispis_2
	);

	process (clk,data_in, start, kraj, reset)
	
		variable zaglavlje:std_logic_vector (191 downto 0):= (others => '0');
		variable zaglavlje2:std_logic_vector (127 downto 0):= (others => '0'); -- koristim zbog potrebe citanja samo drugog dela zaglavlja koji ima 32 bita
		variable total_length:std_logic_vector (15 downto 0);
		variable duz_pak:std_logic_vector (15 downto 0);
		variable zbir:std_logic_vector (19 downto 0) := (others => '0');--za proveru checksume
		variable novi_zbir:std_logic_vector (15 downto 0):= (others => '0');--za proveru checksume
		variable pom_zbir:std_logic_vector (7 downto 0) := (others => '0');--za proveru checksume
		variable novi_pom_zbir:std_logic_vector (7 downto 0):= (others => '0');--za proveru checksume
		variable merac_takta: std_logic_vector (9 downto 0):= (others => '0')  ; -- vreme pristizanja fragmenta/paketa
		variable f:std_logic_vector (1 downto 0); -- numeracija dela paketa
		variable adr_mem:std_logic_vector (9 downto 0); -- adresa za upis i ispis memorije
		variable id_paketa:std_logic_vector (11 downto 0); -- za proveru da li dva fragmenta cine jednu celinu
		variable nastavi:std_logic;

	begin
		if (merac_takta="1111111111") then
		merac_takta:="0000000000";
		else
		merac_takta:=merac_takta+1;
		end if;
		if(reset='1') then
			i<=0;
			upis_paket<='0';
		else
			if(clk' event and clk='1') then	
				if(i=0) then upis_paket<='0'; upis_fifo_cek<='0'; end if;
				if(start='1') then
					zaglavlje(191 downto 64):=data_in;
					i<=1;	
					
				end if;
			
				if(i=1) then
					zaglavlje2:=data_in;
					zaglavlje(63 downto 0):=zaglavlje2(127 downto 64);
					if (zaglavlje(191 downto 188)="0100") then
						if( zaglavlje(127 downto 120)/="00000000") then
							duz_pak:=zaglavlje(175 downto 160)-2;--podatak koji nam govori kolika je duzina paketa
							total_length:=duz_pak;
							--provera checksume a samim tim i provera zaglavlja
							zbir:=("0000" & zaglavlje(191 downto 176))+zaglavlje(175 downto 160)+zaglavlje(159 downto 144)+zaglavlje(143 downto 128)+zaglavlje(127 downto 112)+zaglavlje(111 downto 96)+zaglavlje(95 downto 80)+zaglavlje(79 downto 64)+zaglavlje(63 downto 48)+zaglavlje(47 downto 32);
							novi_zbir:=zbir(15 downto 0)+zbir(19 downto 16);
							pom_zbir:=zaglavlje(31 downto 24)+zaglavlje(23 downto 16)+zaglavlje(15 downto 8); --novo
							novi_pom_zbir:=pom_zbir(7 downto 0)+zaglavlje(7 downto 0); --novo
							f:=zaglavlje(31 downto 30);
							adr_mem:=zaglavlje(29 downto 20);
							id_paketa:=zaglavlje(19 downto 8);
							if(novi_zbir=x"ffff" and novi_pom_zbir=x"ff") then --proveravaju se validnosti obe check_sum-e
							
								if (f="00") then							
										i<=2; -- ukoliko je ceo paket
										
								else
										ka_fifo_cekanje<=conv_std_logic_vector(0,78) & adr_mem & id_paketa & merac_takta & f & duz_pak ;
										upis_fifo_cek<='1';
										i<=3; -- logika ukoliko imamo fragment
								end if;
								
							else
									i<=0;--kada zaglavlje nije dobro ne salje se nista na izlaz i postavaljamo i=0
							end if;-- novi zbir
						end if;--da li je TTL polje razlicito od 0
					end if;-- da li je verzija IPv4
				end if;-- kada je i=2
			
				if(i=2) then
					
					if(duz_pak=total_length) then
						ka_fifo_paket<="10" & data_in; -- pocetak paketa
						duz_pak:=duz_pak-1;
						upis_paket<='1';
					else
						ka_fifo_paket<="00" & data_in; -- sredina paketa
						duz_pak:=duz_pak-1;
					end if;
					if(kraj='1') then
							ka_fifo_paket<="01" & data_in; -- kraj paketa
							
							i<=0;
					end if;
				end if;--kada je i>=3
				
				if(i=3) then
					ka_fifo_cekanje<=data_in;
					if(kraj='1') then
							i<=0; -- kada je poslat i poslednji deo fragmenta
						
					end if;
				end if;--kada je i>=4
			end if;--clk
		end if; --reset
	end process;
	
	process (clk, reset)
		variable pom:std_logic_vector (1023 downto 0):= (others => '0');
		variable zaglavlje_pom:std_logic_vector (39 downto 0);
		variable zaglavlje_cek:std_logic_vector (49 downto 0);
		variable iz_memorije:std_logic_vector (1151 downto 0);
		variable iz_memorije_pom:std_logic_vector (1151 downto 0):= (others => '0');
		variable u_memoriju_temp:std_logic_vector (1151 downto 0):= (others => '0');
		variable u_memoriju_pom:std_logic_vector (1151 downto 0):= (others => '0');
		variable nastavi:std_logic:='0';
		variable fr1:std_logic_vector (11 downto 0);
		variable fr2:std_logic_vector (11 downto 0);
		variable duz_fr1:std_logic_vector (15 downto 0);
		variable duz_fr2:std_logic_vector (15 downto 0);
		variable duz_fr1_max:std_logic_vector (15 downto 0);
		variable duz_fr2_max:std_logic_vector (15 downto 0);
		variable tren1:std_logic_vector (9 downto 0);
		variable tren2:std_logic_vector (9 downto 0);
		variable f1:std_logic_vector (1 downto 0);
		variable f2:std_logic_vector (1 downto 0);
		variable pocetak:integer;
		variable pomeraj:std_logic_vector (1023 downto 0):= (others => '0');
		variable odbacivanje:std_logic_vector (127 downto 0);
		variable za_upis_pomeraj:std_logic_vector(1023 downto 0):= (others => '0');
		variable kraj:integer;
		variable adresa:integer;
		variable nastavak:std_logic:='0';
	begin
	
		if(reset='1') then
			j<=0;
		else
		
			if(clk' event and clk='1') then
				dalje_fifo_cek<='0';
				upis_fragment<='0';
					if (j=0) then
						if(prazan_fifo_cek='0') then
						upisi_u_prvu<="0";
						upisi_u_drugu<="0";
						dalje_fifo_cek<='1'; -- za ovaj podatak su potrebna 2 takta
						if (nastavi='1') then -- na ovaj nacin dodajemo jos jedan takt pre nego sto predjemo na novi korak
							j<=1;
						dalje_fifo_cek<='0';
						nastavi:='0';
						else
							nastavi:='1';
						end if;
					end if;
					end if;
					if(j=1) then
						
						-- prikupljamo potrebne podatke iz fifo_fragment_cekanje--
						adresa:=conv_integer(iz_fifo_cek(49 downto 40));
						zaglavlje_cek:=iz_fifo_cek(49 downto 0);
						u_memoriju_pom:=conv_std_logic_vector(0,1152);
						u_memoriju_temp:=conv_std_logic_vector(0,1152);
						fr1:=iz_fifo_cek(39 downto 28);
						tren1:=iz_fifo_cek(27 downto 18);
						f1:=iz_fifo_cek(17 downto 16);				
						duz_fr1:=iz_fifo_cek(15 downto 0);
						duz_fr1_max:=duz_fr1;
						if (pom(adresa)='0') then -- ukoliko u RAM memoriji ne postoji nijedan podatak upisan na toj adresi ide se na bezuslovni upis u nju
							dalje_fifo_cek<='1';
							j<=5;
						else -- u suprotnom cita se podatak iz naznacene adrese i prelazi na sledeci korak
						j<=2;
						adr_mem_ispis_1<=iz_fifo_cek(49 downto 40);
						adr_mem_ispis_2<=iz_fifo_cek(49 downto 40);
						end if;
					end if;
					if (j=2) then
					if(nastavak='0') then
					
					nastavak:='1';
					else
					-- prikupljanje podataka iz prve memorije--
					iz_memorije:=iz_druge_memorije;
					zaglavlje_pom:=iz_prve_memorije;
					fr2:=zaglavlje_pom(39 downto 28);
					f2:=zaglavlje_pom(17 downto 16);
					tren2:=zaglavlje_pom(27 downto 18);
					nastavak:='0';
					duz_fr2:=zaglavlje_pom(15 downto 0);
					duz_fr2_max:=duz_fr2;
						if(fr1=fr2) then -- da li su dva fragmenta deo jedne celine		
							if(f1/=f2) then -- da li su duplikat
								if(f1="01") then --fragment iz fifo_fragment_cekanje je prvi deo paketa								
									dalje_fifo_cek<='1';
									j<=3;
								else --fragment iz RAM memorije je prvi deo paketa
									
									j<=4;
								end if;
							else j<=6;
							end if;
						else
						
							if(tren1-tren2>50) then --provera "starosti" paketa u memoriji
								dalje_fifo_cek<='1';
								j<=5;
							else
								upisi_u_prvu<="1";
								upisi_u_drugu<="1";
								u_prvu_memoriju<=zaglavlje_pom;
								u_drugu_memoriju<=iz_memorije;
								j<=6;
								end if;
						end if;
						end if;
					end if;					
					if (j=3) then -- kada je iz fifo_fragment_cekanje prvi deo paketa				
						if (nastavi='1')	then	
							if (duz_fr1>0) then
								if (duz_fr1=duz_fr1_max) then
									ka_fifo_fragment<="10" & iz_fifo_cek; -- pocetak paketa
									upis_fragment<='1';
									dalje_fifo_cek<='1';
									duz_fr1:=duz_fr1-1;
								else
									ka_fifo_fragment<="00" & iz_fifo_cek; --sredina paketa
									upis_fragment<='1';
									dalje_fifo_cek<='1';
									duz_fr1:=duz_fr1-1;
								end if;
								
							else -- podaci iz RAM memorije
								if (duz_fr2>1) then
									ka_fifo_fragment<="00" & iz_memorije(1151 downto 1024); -- sredina paketa
									upis_fragment<='1';
									pomeraj:=iz_memorije(1023 downto 0); --shift-ovanje fragmenta
									iz_memorije(1151 downto 128):=pomeraj;
									duz_fr2:=duz_fr2-1;
									dalje_fifo_cek<='0';
								else
									ka_fifo_fragment<="01" & iz_memorije(1151 downto 1024); --kraj paketa
									upis_fragment<='1';
									dalje_fifo_cek<='0';
									pom(adresa):='0';
									nastavi:='0';
									j<=0;
								end if;
							end if;
						else
							nastavi:='1'; --kasnjenje zbog vremena potrebnog da se iscita iz fifo_fragment_cekanje
							dalje_fifo_cek<='1';
						end if;
					end if;

					if (j=4) then --kada je fragment iz RAM memorije prvi deo paketa
						if (duz_fr2<=2) then dalje_fifo_cek<='1'; end if;
						if (duz_fr2>=1) then
						if (duz_fr2=duz_fr2_max) then
						ka_fifo_fragment<="10" & iz_druge_memorije(1151 downto 1024); --pocetak paketa
									upis_fragment<='1';
									pomeraj:=iz_druge_memorije(1023 downto 0); --shift-ovanje fragmenta
									iz_memorije(1151 downto 128):=pomeraj;
									duz_fr2:=duz_fr2-1;
						else
							ka_fifo_fragment<="00" & iz_memorije(1151 downto 1024); --sredina paketa
							upis_fragment<='1';
							pomeraj:=iz_memorije(1023 downto 0); --shiftovanje fragmenta
							iz_memorije(1151 downto 128):=pomeraj;
							pom(adresa):='0';
							duz_fr2:=duz_fr2-1;
						end if;
						else
							if (duz_fr1>1) then
								ka_fifo_fragment<="00" & iz_fifo_cek; -- sredina paketa
								upis_fragment<='1';
								duz_fr1:=duz_fr1-1;
								dalje_fifo_cek<='1';
								if (duz_fr1<=1) then dalje_fifo_cek<='0'; end if;
							else
								ka_fifo_fragment<="01" & iz_fifo_cek; --kraj paketa
								upis_fragment<='1';
								dalje_fifo_cek<='0';
								j<=0;
							end if;
						end if;
					end if;

					if (j=5) then --kada iz fifo_fragment_cekanje treba da se upise fragment u memoriju
						if (nastavi='1') then
							if (duz_fr1>1) then
								za_upis_pomeraj:=u_memoriju_pom(1023 downto 0);
								u_memoriju_pom:=za_upis_pomeraj & iz_fifo_cek; --upis iz fifo_fragment_cekanje uz shift-ovanje fragmenta
								duz_fr1:=duz_fr1-1;
								dalje_fifo_cek<='1';
								if (duz_fr1=1) then
									dalje_fifo_cek<='0';
								end if;
							else
								za_upis_pomeraj:=u_memoriju_pom(1023 downto 0);
								u_memoriju_pom:=za_upis_pomeraj & iz_fifo_cek;
								if (duz_fr1_max<9) then -- ukoliko fragment nije maksimalne duzine
								pocetak:=conv_integer(duz_fr1_max)*128-1;
								kraj:=1151-pocetak;
								u_memoriju_temp(1151 downto kraj):=u_memoriju_pom(pocetak downto 0);
								u_memoriju_pom:=u_memoriju_temp;
								end if;
								--upis u memoriju--
								adr_mem_upis_1<=zaglavlje_cek(49 downto 40);
								adr_mem_upis_2<=zaglavlje_cek(49 downto 40);
								upisi_u_prvu<="1";
								upisi_u_drugu<="1";
								u_prvu_memoriju<=zaglavlje_cek(39 downto 0);
								u_drugu_memoriju<=u_memoriju_pom;
								pom(adresa):='1';
								j<=0;
								nastavi:='0';

							end if;
						else nastavi:='1';
						dalje_fifo_cek<='1';
						end if;
					end if;


					if (j=6) then --odbacivanje fragmenta iz fifo_fragment_cekanje
						upisi_u_prvu<="0";
						upisi_u_drugu<="0";
						if (duz_fr1>0) then

							odbacivanje:=iz_fifo_cek;
							duz_fr1:= duz_fr1-1;
							if (duz_fr1=2) then
								dalje_fifo_cek<='0';
							end if;

						else
							j<=0;
						end if;

					end if;
				--end if;
			end if;
		end if;
	end process;
	
	process (clk)
	
	variable robin: std_logic_vector (1 downto 0):="01"; --round robin
	variable dalje_fragment: std_logic := '0';
	variable dalje_paket: std_logic := '0';
	variable jos_fragment: std_logic:= '0';
	variable jos_paket: std_logic:= '0';
	variable start_kraj: std_logic_vector (1 downto 0) := "00";
	variable dalje_pom:std_logic:='0';
	variable procitano:std_logic_vector (129 downto 0) ;
	
begin
	
		if(clk' event and clk='1') then
				start_out<='0';
				kraj_out<='0';
				citaj_paket<='0';
				citaj_fragment<='0';
				data_out<=(others=>'0');
			if(robin="00") then --medjufaza za robin="10"
			if (dalje_pom='1') then
			robin:="10";
			dalje_pom:='0';
			else
			dalje_pom:='1';
			citaj_fragment<='1';
			end if;
			end if;
			
			
			if (robin="01") then -- citanje iz fifo_paket
			if (prazan_fifo_paket='1' and jos_paket='0') then --ukoliko je fifo_paket prazan prelazi se na fifo_fragment
			robin:="10";
			citaj_fragment<='1';
			else
				if (dalje_paket='1') then
					jos_paket:='1'; --kad se isprazni fifo_paket naznaka da ima jos jedan deo paketa da se iscita
					citaj_paket<='1';
					procitano:=procitano_paket;
					start_kraj:=procitano(129 downto 128);
					data_out<=procitano(127 downto 0);
					if (start_kraj="10") then -- znak za start paketa
						start_out<='1';
						kraj_out<='0';
					end if;
					if (start_kraj="01") then --znak za kraj paketa i prelazak na fifo_fragment
						kraj_out<='1';
						start_out<='0';
						citaj_paket<='0';
						robin:="10";
						citaj_fragment<='1';
						jos_paket:='0';
						dalje_paket:='0';
					end if;
					if (start_kraj="00") then --sredina paketa
						kraj_out<='0';
						start_out<='0';
					end if;
				else --zakasnjenje za citanje iz fifo_paketa
					dalje_paket:='1';
					robin:="11"; --prelazak u medjufazu
				end if;
				end if;
			end if;

	
			if (robin="10") then --citanje iz fifo_fragment
				if(prazan_fifo_fragment='1' and jos_fragment='0') then -- ukoliko je fifo_fragment prazan prelazi se na fifo_paket 
				robin:="01";
				citaj_fragment<='0';
				else
				if (dalje_fragment='1') then
					jos_fragment:='1'; --kad se isprazni fifo_fragment naznaka da ima jos jedan deo paketa da se iscita
					procitano:=procitano_fragment;
					start_kraj:=procitano(129 downto 128);
					data_out<=procitano(127 downto 0);
					citaj_fragment<='1';
					if (start_kraj="10") then --znak za start paketa
						start_out<='1';
						kraj_out<='0';
					end if;
					if (start_kraj="01") then --znak za kraj paketa i prelazak na fifo_paket
						kraj_out<='1';
						start_out<='0';
						jos_fragment:='0';
						citaj_fragment<='0';
						robin:="01";
						dalje_fragment:='0';
					end if;
					if (start_kraj="00") then --sredina paketa
						kraj_out<='0';
						start_out<='0';
					end if;
				else
					dalje_fragment:='1';
					robin:="00"; --prelazak u medjufazu
				end if;
				end if;
			end if;
		if(robin="11") then --medjufaza za robin="01"
			if (dalje_pom='1') then
			robin:="01";
			citaj_paket<='1'; -- zato sto je na kraju konfiguracije neophodan korak za pravilno citanje
			dalje_pom:='0';
			else
			dalje_pom:='1';
			citaj_paket<='1';
			end if;
			end if;
			
		end if;
	end process;
	end shema;