library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity data_enkripcija is
	Port 
	( 
		clk 			 			: in    STD_LOGIC;
		start			 			: in    STD_LOGIC;
		kraj						: in    STD_LOGIC;
		izvorisna_adresa		: in    STD_LOGIC_VECTOR (31 downto 0);
		odredisna_adresa		: in    STD_LOGIC_VECTOR (31 downto 0);
		data_in	 	 			: in    STD_LOGIC_VECTOR (127 downto 0);
		kljuc0_in 	 			: in    STD_LOGIC_VECTOR (127 downto 0);
		kljuc1_in 	 			: in    STD_LOGIC_VECTOR (127 downto 0);
		kljuc2_in 	 			: in    STD_LOGIC_VECTOR (127 downto 0);
		kljuc3_in 	 			: in    STD_LOGIC_VECTOR (127 downto 0);
		kljuc4_in 	 			: in    STD_LOGIC_VECTOR (127 downto 0);
		kljuc5_in 	 			: in    STD_LOGIC_VECTOR (127 downto 0);
		kljuc6_in 	 			: in    STD_LOGIC_VECTOR (127 downto 0);
		kljuc7_in 	 			: in    STD_LOGIC_VECTOR (127 downto 0);
		kljuc8_in 	 			: in    STD_LOGIC_VECTOR (127 downto 0);
		kljuc9_in 	 			: in    STD_LOGIC_VECTOR (127 downto 0);
		kljuc10_in	 			: in    STD_LOGIC_VECTOR (127 downto 0);
		data_out		 			: out   STD_LOGIC_VECTOR (127 downto 0);
		izvorisna_adresa_out	: out   STD_LOGIC_VECTOR (31 downto 0);
		odredisna_adresa_out	: out   STD_LOGIC_VECTOR (31 downto 0);
		start_out	 			: out	  STD_LOGIC;
		kraj_out		 			: out	  STD_LOGIC
	);
end data_enkripcija;

architecture shema of data_enkripcija is

component prva_runda_Tx is
	Port 
	( 
		data_in  : in   STD_LOGIC_VECTOR (127 downto 0);
		kljuc_in : in   STD_LOGIC_VECTOR (127 downto 0);
		data_out : out  STD_LOGIC_VECTOR (127 downto 0)
	);
end component;

component runda_Tx is
	Port 
	( 
		data_in  : in   STD_LOGIC_VECTOR (127 downto 0);
		kljuc_in : in   STD_LOGIC_VECTOR (127 downto 0);
		data_out : out  STD_LOGIC_VECTOR (127 downto 0)
	);
end component;

component poslednja_runda_Tx is
	Port 
	( 
		data_in  : in   STD_LOGIC_VECTOR (127 downto 0);
		kljuc_in : in   STD_LOGIC_VECTOR (127 downto 0);
		data_out : out  STD_LOGIC_VECTOR (127 downto 0)
	);
end component;


type start_kraj is array (0 to 11) of std_logic; -- novi tip kojim se vrsi pomeranje signala start i kraj radi sinhornizacije
signal startt, krajj : start_kraj;							-- sa podatkom

type adrese is array (0 to 11) of std_logic_vector(31 downto 0); -- za potrebe sinhronizacije ovih signal, posto nam je potrebno da i oni
signal izvorisna_adr, odredisna_adr : adrese;			-- budu na izlazu kada i prvi paket

type niz_registara is array (10 downto 0) of std_logic_vector(127 downto 0);
signal kljucevi, data_inn, data_outt: niz_registara;

signal data_in_reg :std_logic_vector (127 downto 0); --koristimo za potrebe pomeranja glavnog paketa
begin
	
	prva_inst: prva_runda_Tx
	port map
	(
		data_in => data_inn(0),
		kljuc_in => kljucevi(0),
		data_out => data_outt(0)
	);
	
	generisi_runde:FOR i IN 1 TO 9 GENERATE
		runda_inst: runda_Tx
		port map
		(
			data_in  => data_inn(i),
			kljuc_in => kljucevi(i),
			data_out => data_outt(i)				
		);
	END GENERATE;
	
	poslednja_inst: poslednja_runda_Tx
	port map
	(
			data_in  => data_inn(10),
			kljuc_in => kljucevi(10),
			data_out => data_outt(10)		
	);	
	-- povezivanje tipa kljucevi sa odgovarajuce kljucevima sa ulaza
	kljucevi(0)<=kljuc0_in;
	kljucevi(1)<=kljuc1_in;
	kljucevi(2)<=kljuc2_in;
	kljucevi(3)<=kljuc3_in;
	kljucevi(4)<=kljuc4_in;
	kljucevi(5)<=kljuc5_in;
	kljucevi(6)<=kljuc6_in;
	kljucevi(7)<=kljuc7_in;
	kljucevi(8)<=kljuc8_in;
	kljucevi(9)<=kljuc9_in;
	kljucevi(10)<=kljuc10_in;

	process(clk,data_in)
		
		variable data2_in :std_logic_vector (127 downto 0):=(others => '0');
		
	begin
	
		if(clk' event and clk='1') then         
			data_in_reg<=data_in;	--zbog kasnjenja signala
			data2_in:=data_in_reg;
			
			data_inn(0)<=data2_in; -- sam nacin poveyivanje entitete prva_runda_Tx, runda_Tx i poslednja_runda_Tx
			for i in 1 to 10 loop
				data_inn(i)<=data_outt(i-1);
			end loop;
			data_out<=data_outt(10);
			
			-- nacini pomeranja signal da bi dosli u istom trenutku na izlaz kad i prvi podataka
			startt(0)<=start; 
			krajj(0)<=kraj;
			for i in 1 to 11 loop
				startt(i)<=startt(i-1);
				krajj(i)<=krajj(i-1);
			end loop;
			start_out<=startt(11);
			kraj_out<=krajj(11);
			
			izvorisna_adr(0)<=izvorisna_adresa;
			odredisna_adr(0)<=odredisna_adresa;
			for i in 1 to 11 loop
				izvorisna_adr(i)<=izvorisna_adr(i-1);
				odredisna_adr(i)<=odredisna_adr(i-1);
			end loop;
			izvorisna_adresa_out<=izvorisna_adr(11);
			odredisna_adresa_out<=odredisna_adr(11);
			
			
			
		end if;-- if od clk

	end process;
	
end shema;

